const {
  csv,
  select,
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
} = d3;

const csvUrl =
  'https://gist.githubusercontent.com/Akhillenkalapalli/4fe5d42a35f9edbba5d5d7009698e10a/raw/03f13d4ce9ccc01dd151a261e9f192e597b8f305/TestCricket_Records.csv';

const parseRow = (d) => ({
  player: d.Player,
  country: d.Country,
  span: d.Span,
  matches: +d.Matches,
  innings: +d.Innings,
  notOuts: +d.NotOuts,
  runs: +d.Runs,
  highestScore: d['Highest Score'],
  average: +d.Average,
  ballsFaced: +d['Balls Faced'],
  strikeRate: +d['Strike Rate'],
  centuries: +d['100'],
  halfCenturies: +d['50'],
  ducks: +d.Ducks,
  fours: +d.Fours,
  sixes: +d.Sixes,
});

const xValue = (d) => d.matches;
const yValue = (d) => d.runs;
const rxValue = (d) => d.innings;
const ryValue = (d) => d.average;

const margin = { top: 30, right: 50, bottom: 50, left: 90 };
const radius = 5;
const width =
  window.innerWidth - margin.left - margin.right;
const height =
  window.innerHeight - 110 - margin.top - margin.bottom;

const svg = select('#scatter-plot')
  .attr('width', width + margin.left + margin.right)
  .attr('height', height + margin.top + margin.bottom)
  .append('g')
  .attr(
    'transform',
    `translate(${margin.left},${margin.top})`,
  );

const main = async () => {
  const data = await csv(csvUrl, parseRow);
  const x = scaleLinear()
    .domain(extent(data, xValue))
    .nice()
    .range([0, width]);
  const y = scaleLinear()
    .domain(extent(data, yValue))
    .nice()
    .range([height, 0]);

  // Assigning colors based on countries
  const colorScale = d3.scaleOrdinal(d3.schemeCategory10);

  const marks = data.map((d) => ({
    x: x(xValue(d)),
    y: y(yValue(d)),
    player: d.player,
    country: d.country,
    runs: d.runs,
    average: d.average,
    rx: rxValue(d),
    ry: ryValue(d),
    color: colorScale(d.country),
  }));

  svg
    .selectAll('.circle')
    .data(marks)
    .enter()
    .append('circle')
    .attr('class', 'circle')
    .attr('cx', (d) => d.x)
    .attr('cy', (d) => d.y)
    .attr('fill', (d) => d.color)
    .attr('r', radius);

  svg
    .append('g')
    .attr('class', 'axis')
    .attr('transform', `translate(0,${height})`)
    .call(axisBottom(x));

  svg
    .append('text')
    .attr('class', 'axis-label')
    .attr('x', width / 2)
    .attr('y', height + margin.bottom / 1.2)
    .text('Matches Played');

  svg.append('g').attr('class', 'axis').call(axisLeft(y));

  svg
    .append('text')
    .attr('class', 'axis-label')
    .attr('transform', 'rotate(-90)')
    .attr('x', -height / 2)
    .attr('y', -margin.left / 1.5)
    .text('Runs');
};

main();
